# -*- coding: utf-8 -*-
# template 18
"""
Python supplemental features
----------------------------

python_plus adds various features to python 2 and python 3 programs.
It is designed to be used as integration of pypi future to help to port
your code from Python 2 to Python 3 and still have it run on Python 2.


vem: virtual environment manager
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

This package is released with an nice command:
**vem** that is an interactive tool with some nice features to manage
standard virtual environment and it is osx/darwin compatible.
"""
import os
import sys
import pkg_resources
import gzip
import shutil


__version__ = '1.0.11'


def fake_setup(**kwargs):
    globals()['setup_args'] = kwargs


def read_setup():
    setup_info = os.path.abspath(
        os.path.join(os.path.dirname(__file__), 'setup.info'))
    if not os.path.isfile(setup_info):
        setup_info = os.path.abspath(
            os.path.join(os.path.dirname(__file__), '..', 'setup.py'))
    setup_args = {}
    if os.path.isfile(setup_info):
        with open(setup_info, 'r') as fd:
            exec(fd.read().replace('setup(', 'fake_setup('))
            setup_args = globals()['setup_args']
    else:
        print('Not internal configuration file found!')
    setup_args['setup'] = setup_info
    try:
        pkg = pkg_resources.get_distribution(__package__.split('.')[0])
        setup_args['name'] = pkg.key
        setup_args['version'] = pkg.version
    except BaseException:
        pass
    return setup_args


def get_pypi_paths():
    local_venv = '/devel/venv/'
    pkgpath = os.path.abspath(
        os.path.join(os.path.dirname(__file__), '..'))
    bin_path = lib_path = ''
    path = pkgpath
    while not bin_path and path != '/' and path != os.environ['HOME']:
        path = os.path.dirname(path)
        if os.path.isdir(path) and os.path.basename(path) == 'lib':
            bin_path = os.path.join(os.path.dirname(path), 'bin')
            lib_path = path
    if not bin_path and local_venv:
        for path in sys.path:
            if local_venv in path:
                bin_path = os.path.join(
                    path[:path.find(local_venv)],
                    *[x for x in local_venv.split('/') if x][:-1])
                break
    return pkgpath, bin_path, lib_path


def copy_pkg_data(setup_args, verbose):
    if setup_args.get('package_data'):
        pkgpath, bin_path, lib_path = get_pypi_paths()
        if bin_path:
            # TODO> compatibility mode
            bin2_path = os.path.join(os.environ['HOME'], 'devel')
            if not os.path.isdir(bin2_path):
                bin2_path = ''
            man_path = os.path.join(bin_path, 'man', 'man8')
            if not os.path.isdir(man_path):
                man_path = ''
            for pkg in setup_args['package_data'].keys():
                for fn in setup_args['package_data'][pkg]:
                    base = os.path.basename(fn)
                    if base in ('setup.info', '*'):
                        continue
                    full_fn = os.path.abspath(os.path.join(pkgpath, fn))
                    if base.endswith('.man') and man_path:
                        with open(full_fn, 'r') as fd:
                            help_text = fd.read()
                        tgt_fn = os.path.join(man_path, '%s.8.gz' % base[:-4])
                        with gzip.open(tgt_fn, 'w') as fd:
                            if sys.version_info[0] == 3:
                                fd.write(help_text.encode('utf-8'))
                            else:
                                fd.write(help_text)
                        continue
                    if lib_path:
                        tgt_fn = os.path.join(lib_path, base)
                        if verbose:
                            print('$ cp %s %s' % (full_fn, tgt_fn))
                        shutil.copy(full_fn, tgt_fn)
                    # TODO> compatibility mode
                    tgt_fn = os.path.join(bin_path, base)
                    if os.path.isfile(tgt_fn):
                        os.unlink(tgt_fn)
                    if not os.path.exists(tgt_fn):
                        if verbose:
                            print('$ ln -s %s %s' % (full_fn, tgt_fn))
                        os.symlink(full_fn, tgt_fn)
                    if bin2_path:
                        tgt_fn = os.path.join(bin2_path, base)
                        if os.path.isfile(tgt_fn):
                            os.unlink(tgt_fn)
                        # if not os.path.exists(tgt_fn):
                        #     if verbose:
                        #         print('$ ln -s %s %s' % (full_fn, tgt_fn))
                        #     os.symlink(full_fn, tgt_fn)
            # TODO> compatibility mode to remove early
            if lib_path and bin2_path:
                for base in ('z0librc', 'odoorc', 'travisrc'):
                    full_fn = os.path.join(bin2_path, base)
                    tgt_fn = os.path.join(bin_path, base)
                    if os.path.exists(full_fn) and not os.path.exists(tgt_fn):
                        if verbose:
                            print('$ cp %s %s' % (full_fn, tgt_fn))
                        shutil.copy(full_fn, tgt_fn)


def main(cli_args=None):
    if not cli_args:
        cli_args = sys.argv[1:]
    action = '-H'
    verbose = False
    for arg in cli_args:
        if arg in ('-h', '-H', '--help', '-V', '--version', '--copy-pkg-data'):
            action = arg
        elif arg == '-v':
            verbose = True
    setup_args = read_setup()
    if action == '-h':
        print('%s [-h][-H][--help][-V][--version][-C][--copy-pkg-data]' %
              setup_args['name'])
    elif action in ('-V', '--version'):
        if setup_args['version'] == __version__:
            print(setup_args['version'])
        else:
            print('Version mismatch %s/%s' % (setup_args['version'],
                                              __version__))
    elif action in ('-H', '--help'):
        for text in __doc__.split('\n'):
            print(text)
    elif action in ('-C', '--copy-pkg-data'):
        copy_pkg_data(setup_args, verbose)
    return 0


# -*- coding: utf-8 -*-
# Copyright (C) 2018-2020 SHS-AV s.r.l. (<http://www.zeroincombenze.org>)
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl).
"""
Base library for continuous test of Odoo apps
This module is Odoo version independent and may be used outside Odoo app (i.e.
with odoorc package).
For this reason, this software does not contain functions with sql call, like
search or browse.

Functions with sql call are in test_common.py file.
"""

from __future__ import print_function, unicode_literals
# from past.builtins import basestring
from python_plus import unicodes

import os
# import sys
import base64
import csv
from openpyxl import load_workbook

__version__ = "1.0.11"


class Z0bugOdoo(object):

    def __init__(self, release=None):
        try:
            import odoo.release as release
            self.release = release
        except ImportError:
            try:
                import openerp.release as release
                self.release = release
            except ImportError:
                self.release = None

    def get_image_filename(self, xref):
        file_image = os.path.join(
            os.path.dirname(__file__),
            'data',
            '%s.png' % xref)
        if os.path.isfile(file_image):
            return file_image
        return False

    def get_image(self, xref):
        file_image = self.get_image_filename(xref)
        if file_image:
            with open(file_image, 'rb') as fd:
                image = fd.read()
            return base64.b64encode(image)
        return False

    def save_row(self, model, row):
        if 'id' in row:
            for field in row.copy().keys():
                if row[field] == r'\N':
                    del row[field]
                elif row[field] == r'\\N':
                    row[field] = r'\N'
            getattr(self, model)[row['id']] = unicodes(row)

    def get_data_file_xlsx(self, model, full_fn):
        pymodel = model.replace('.', '_')
        wb = load_workbook(full_fn)
        for sheet in wb:
            break
        colnames = []
        for column in sheet.columns:
            colnames.append(column[0].value)
        hdr = True
        for line in sheet.rows:
            if hdr:
                hdr = False
                setattr(self, pymodel, {})
                continue
            row = {}
            for column, cell in enumerate(line):
                row[colnames[column]] = cell.value
            self.save_row(pymodel, row)

    def get_data_file_csv(self, model, full_fn):
        pymodel = model.replace('.', '_')
        with open(full_fn, 'r') as fd:
            hdr = True
            csv_obj = csv.DictReader(fd,
                                     fieldnames=[],
                                     restkey='undef_name')
            for row in csv_obj:
                if hdr:
                    hdr = False
                    csv_obj.fieldnames = row['undef_name']
                    setattr(self, pymodel, {})
                    continue
                self.save_row(pymodel, row)

    def get_data_file(self, model, filename):
        full_fn = os.path.join(os.path.dirname(__file__), 'data', filename)
        if os.path.isfile('%s.xlsx' % full_fn):
            return self.get_data_file_xlsx(model, '%s.xlsx' % full_fn)
        else:
            return self.get_data_file_csv(model, '%s.csv' % full_fn)

    def get_test_xrefs(self, model):
        """Return model xref list"""
        pymodel = model.replace('.', '_')
        if not hasattr(self, pymodel):
            self.get_data_file(model, pymodel)
        return list(getattr(self, pymodel))

    def get_test_values(self, model, xref):
        """Return model values for test"""
        xids = xref.split('.')
        if len(xids) == 1:
            xids[0], xids[1] = 'z0bug', xids[0]
        pymodel = model.replace('.', '_')
        if not hasattr(self, pymodel):
            self.get_data_file(model, pymodel)
        if xref not in getattr(self, pymodel):
            if xids[0] == 'z0bug':
                raise KeyError('Invalid xref %s for model %s!' % (xref, model))
            return {}
        return getattr(self, pymodel)[xref]

    def initialize_model(self, model):
        """Write all record of model with test values"""
        pymodel = model.replace('.', '_')
        if not hasattr(self, pymodel):
            self.get_data_file(model, pymodel)
        for xref in getattr(self, pymodel):
            pass


#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import print_function, unicode_literals
import os
import sys
try:
    import ConfigParser
except ImportError:
    import configparser as ConfigParser
try:
    from clodoo import clodoo
except ImportError:
    import clodoo
if sys.version_info[0] == 2:
    from travis_helpers import print_flush
else:
    from .travis_helpers import print_flush


def main(argv=None, database=None):
    odoo_version = os.environ.get("VERSION")
    if int(odoo_version.split('.')[0]) < 10:
        src_confn = os.path.expanduser('~/.openerp_serverrc')
    else:
        src_confn = os.path.expanduser('~/.odoorc')
    fname_conf = os.path.expanduser('./clodoo.conf')
    config = ConfigParser.ConfigParser()
    config.read(src_confn)
    if not config.has_section('options'):
        config.add_section('options')
    data = {
        'login_user': 'admin',
        'psycopg2': '1',
        'oe_version': odoo_version,
    }
    for key, value in data.items():
        config.set('options', key, value)
    with open(fname_conf, 'w') as configfile:
        config.write(configfile)
    ctx = {}
    uid, ctx = clodoo.oerp_set_env(
        confn=fname_conf,
        db=database,
        ctx=ctx)
    if not uid:
        print_flush('ERROR: Cannot connect to DB %s with user %s!' % (
            database, data['login_user'],
        ))
        return 1
    if ctx['_cr']:
        print_flush('ERROR: Cannot connect to DB %s via sql with user %s|' % (
            database, config.get('options', 'db_user')
        ))
        return 1
    query = "select name from ir_module_module where state='installed'"
    rows = clodoo.exec_sql(ctx, query, response=True)
    for row in rows:
        print_flush('Module %s' % row[0])
    return 0


if __name__ == "__main__":
    database = os.environ.get('MQT_TEST_DB', 'openerp_test')
    exit(main(database=database))


# -*- coding: utf-8 -*-
from . import apis
from . import get_test_dependencies
from . import getaddons
from . import git_run
from . import odoo_connection
from . import run_pylint
from . import test_server
from . import travis_helpers
from . import travis_tnlbot



#!/usr/bin/env python
"""
Usage: get-addons [-m] path1 [path2 ...][ -e exclude_modules]
          [ --only-applications ][ --exclude-applications]
          [ --only-localization][ --exclude-localization]
Given a list of paths, finds and returns a list of valid addons paths.
With -m flag, will return a list of modules names instead.
"""
from __future__ import print_function

import ast
import os
import sys

if sys.version_info[0] == 2:
    from git_run import GitRun
else:
    from .git_run import GitRun

__version__ = '1.0.11'

MANIFEST_FILES = [
    '__manifest__.py',
    '__odoo__.py',
    '__openerp__.py',
    '__terp__.py',
]


def is_module(path):
    """return False if the path doesn't contain an odoo module, and the full
    path to the module manifest otherwise"""

    path = os.path.expanduser(path)
    if not os.path.isdir(path):
        return False
    files = os.listdir(path)
    filtered = [x for x in files if x in (MANIFEST_FILES + ['__init__.py'])]
    if len(filtered) == 2 and '__init__.py' in filtered:
        return os.path.join(
            path, next(x for x in filtered if x != '__init__.py'))
    else:
        return False


def get_modules(path, depth=1):
    """ Return modules of path repo (used in test_server.py)"""
    return sorted(list(get_modules_info(path, depth).keys()))


def get_modules_info(path, depth=1):
    """ Return a digest of each installable module's manifest in path repo"""
    # Avoid empty basename when path ends with slash
    path = os.path.expanduser(path)
    if not os.path.basename(path):
        path = os.path.dirname(path)

    modules = {}
    if os.path.isdir(path) and depth > 0:
        for module in os.listdir(path):
            manifest_path = is_module(os.path.join(path, module))
            if manifest_path:
                try:
                    manifest = ast.literal_eval(open(manifest_path).read())
                except ImportError:
                    raise Exception('Wrong file %s' % manifest_path)
                if manifest.get('installable', True):
                    modules[module] = {
                        'application': manifest.get('application'),
                        'depends': manifest.get('depends') or [],
                        'auto_install': manifest.get('auto_install'),
                    }
            else:
                deeper_modules = get_modules_info(
                    os.path.join(path, module), depth-1)
                modules.update(deeper_modules)
    return modules


def is_addons(path):
    res = get_modules(path) != []
    return res


def get_addons(path, depth=1):
    """Return repositories in path. Can search in inner folders as depth."""
    path = os.path.expanduser(path)
    if not os.path.exists(path) or depth < 0:
        return []
    res = []
    if is_addons(path):
        res.append(path)
    else:
        new_paths = [os.path.join(path, x)
                     for x in sorted(os.listdir(path))
                     if os.path.isdir(os.path.join(path, x))]
        for new_path in new_paths:
            res.extend(get_addons(new_path, depth-1))
    return res


def get_modules_changed(path, ref='HEAD'):
    """Get modules changed from git diff-index {ref}
    :param path: String path of git repo
    :param ref: branch or remote/branch or sha to compare
    :return: List of paths of modules changed
    """
    path = os.path.expanduser(path)
    git_run_obj = GitRun(os.path.join(path, '.git'))
    if ref != 'HEAD':
        fetch_ref = ref
        if ':' not in fetch_ref:
            # to force create branch
            fetch_ref += ':' + fetch_ref
        git_run_obj.run(['fetch'] + fetch_ref.split('/', 1))
    items_changed = git_run_obj.get_items_changed(ref)
    folders_changed = set([
        item_changed.split('/')[0]
        for item_changed in items_changed
        if '/' in item_changed]
    )
    modules = set(get_modules(path))
    modules_changed = list(modules & folders_changed)
    modules_changed_path = [
        os.path.join(path, module_changed)
        for module_changed in modules_changed]
    return modules_changed_path


def get_dependencies(modules, module_name):
    """Return a set of all the dependencies in deep of the module_name.
    The module_name is included in the result."""
    result = set()
    for dependency in modules.get(module_name, {}).get('depends', []):
        result |= get_dependencies(modules, dependency)
    return result | set([module_name])


def get_dependents(modules, module_name):
    """Return a set of all the modules that are dependent of the module_name.
    The module_name is included in the result."""
    result = set()
    for dependent in modules.keys():
        if module_name in modules.get(dependent, {}).get('depends', []):
            result |= get_dependents(modules, dependent)
    return result | set([module_name])


def add_auto_install(modules, to_install):
    """ Append automatically installed glue modules to to_install if their
    dependencies are already present. to_install is a set. """
    found = True
    while found:
        found = False
        for module, module_data in modules.items():
            if (module_data.get('auto_install') and
                    module not in to_install and
                    all(dependency in to_install
                        for dependency in module_data.get('depends', []))):
                found = True
                to_install.add(module)
    return to_install


def get_applications_with_dependencies(modules):
    """ Return all modules marked as application with their dependencies.
    For our purposes, l10n modules cannot be an application. """
    result = set()
    for module, module_data in modules.items():
        if module_data.get('application') and not module.startswith('l10n_'):
            result |= get_dependencies(modules, module)
    return add_auto_install(modules, result)


def get_localizations_with_dependents(modules):
    """ Return all localization modules with the modules that depend on them
    """
    result = set()
    for module in modules.keys():
        if module.startswith('l10n_'):
            result |= get_dependents(modules, module)
    return result


def main(argv=None):
    if argv is None:
        argv = sys.argv
    argv = argv[1:]
    if not argv:
        print(__doc__)
        return 1

    list_modules = False
    application = None
    localization = None
    exclude_modules = []

    params = []
    while argv:
        if argv[0].startswith('-'):
            param = argv.pop(0)
            if param == '-h' or param == '--help':
                print(__doc__)
                return 1
            elif param == '-V' or param == '--version':
                print(__version__)
                return 1
            elif param == '-m':
                list_modules = True
            elif param == '-e':
                exclude_modules = [x for x in argv.pop(0).split(',')]
            elif param == '--only-applications':
                application = True
            elif param == '--exclude-applications':
                application = False
            elif param == '--only-localization':
                localization = True
            elif param == '--exclude-localization':
                localization = False
            else:
                raise Exception('Unknown parameter: %s' % param)
        else:
            params.append(argv.pop(0))

    if list_modules:
        modules = {}
        for path in params:
            modules.update(get_modules_info(path))
        res = set(modules.keys())
        applications, localizations = set(), set()
        if application is True or application is False:
            applications = get_applications_with_dependencies(modules)
            if not application:
                res -= applications
                applications = set()
        if localization is True or localization is False:
            localizations = get_localizations_with_dependents(modules)
            if not localization:
                res -= localizations
                localizations = set()
        if application or localization:
            res = applications | localizations
        res = sorted(list(res))
    else:
        lists = [get_addons(path) for path in params]
        # return flatten list of lists without duplicates
        res = dict.fromkeys([x for lx in lists for x in lx]).keys()
    if exclude_modules:
        res = [x for x in res if x not in exclude_modules]
    print(','.join(res))
    return 0


if __name__ == "__main__":
    sys.exit(main())


#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import print_function

import importlib
import re
import os
import shutil
import subprocess
import sys
from six import string_types
from zerobug import z0testodoo
if sys.version_info[0] == 2:
    from getaddons import (
        get_addons, get_modules, get_modules_info, get_dependencies,
        get_applications_with_dependencies, get_localizations_with_dependents)
    from travis_helpers import success_msg, fail_msg, print_flush
else:
    from .getaddons import (
        get_addons, get_modules, get_modules_info, get_dependencies,
        get_applications_with_dependencies, get_localizations_with_dependents)
    from .travis_helpers import success_msg, fail_msg, print_flush
try:
    import ConfigParser
except ImportError:
    import configparser as ConfigParser

__version__ = '1.0.11'

LDIR = ('server/openerp', 'odoo/odoo', 'openerp', 'odoo')


def has_test_errors(fname, dbname, odoo_version, check_loaded=True):
    """
    Check a list of log lines for test errors.
    Extension point to detect false positives.
    """
    # Rules defining checks to perform
    # this can be
    # - a string which will be checked in a simple substring match
    # - a regex object that will be matched against the whole message
    # - a callable that receives a dictionary of the form
    #     {
    #         'loglevel': ...,
    #         'message': ....,
    #     }
    errors_ignore = [
        'Mail delivery failed',
        'failed sending mail',
        ]
    errors_report = [
        lambda x: x['loglevel'] == 'CRITICAL',
        'At least one test failed',
        'no access rules, consider adding one',
        'invalid module names, ignored',
        'raise exception'
        ]
    # Only check ERROR lines before 7.0
    if int(odoo_version.split('.')[0]) < 7:
        errors_report.append(
            lambda x: x['loglevel'] == 'ERROR')

    def make_pattern_list_callable(pattern_list):
        for i in range(len(pattern_list)):
            if isinstance(pattern_list[i], string_types):
                regex = re.compile(pattern_list[i])
                pattern_list[i] = lambda x, regex=regex: regex.search(
                    x['message'])
            elif hasattr(pattern_list[i], 'match'):
                regex = pattern_list[i]
                pattern_list[i] = lambda x, regex=regex: regex.search(
                    x['message'])

    make_pattern_list_callable(errors_ignore)
    make_pattern_list_callable(errors_report)

    print("-" * 10)
    # Read log file removing ASCII color escapes:
    # http://serverfault.com/questions/71285
    color_regex = re.compile(r'\x1B\[([0-9]{1,2}(;[0-9]{1,2})?)?[m|K]')
    log_start_regex = re.compile(
        r'^\d{4}-\d{2}-\d{2} \d{2}:\d{2}:\d{2},\d{3} \d+ (?P<loglevel>\w+) '
        r'(?P<db>(%s)|([?])) (?P<logger>\S+): (?P<message>.*\S)\s*$' % dbname)
    log_records = []
    last_log_record = dict.fromkeys(log_start_regex.groupindex.keys())
    with open(fname) as log:
        for line in log:
            line = color_regex.sub('', line)
            match = log_start_regex.match(line)
            if match:
                last_log_record = match.groupdict()
                log_records.append(last_log_record)
            else:
                last_log_record['message'] = '%s\n%s' % (
                    last_log_record['message'], line.rstrip('\n')
                )
    errors = []
    for log_record in log_records:
        ignore = False
        for ignore_pattern in errors_ignore:
            if ignore_pattern(log_record):
                ignore = True
                break
        if ignore:
            continue
        for report_pattern in errors_report:
            if report_pattern(log_record):
                errors.append(log_record)
                break

    if check_loaded:
        if not [r for r in log_records if 'Modules loaded.' in r['message']]:
            errors.append({'message': "Message not found: 'Modules loaded.'"})

    if errors:
        for e in errors:
            print(e['message'])
        print("-" * 10)
    return len(errors)


def parse_list(comma_sep_list):
    return [x.strip() for x in comma_sep_list.split(',') if x.strip()]


def str2bool(string):
    return str(string or '').lower() in ['1', 'true', 'yes']


def get_server_path(odoo_full, odoo_version, travis_home):
    """
    Computes server path
    :param odoo_full: Odoo repository path
    :param odoo_version: Odoo version
    :param travis_home: Travis home directory
    :return: Server path
    """
    odoo_version = odoo_version.replace('/', '-')
    odoo_org, odoo_repo = odoo_full.split('/')
    server_dirname = "%s-%s" % (odoo_repo, odoo_version)
    server_path = os.path.join(travis_home, server_dirname)
    return server_path


def get_addons_path(travis_dependencies_dir, travis_base_dir, server_path,
                    odoo_test_select):
    """
    Computes addons path
    :param travis_dependencies_dir: Travis dependencies directory
    :param travis_base_dir: Travis odoo core directory
    :param server_path: Server path
    :param odoo module set to include/exclude
    :return: Addons path
    """
    addons_path_list = []
    for ldir in LDIR:
        if os.path.isdir(os.path.join(server_path, ldir, 'addons')):
            addons_path_list = get_addons(
                os.path.join(server_path, ldir, 'addons'))
            break
    addons_path_list.append(os.path.join(server_path, "addons"))
    addons_path_list.extend(get_addons(travis_base_dir))
    addons_path_list.extend(get_addons(travis_dependencies_dir))
    addons_path = ','.join(addons_path_list)
    return addons_path


# Discover automatically Odoo dir and version
# Recognizes old 6.1 tree, 7.0/8.0/9.0 tree and new 10.0/../15.0 tree
def get_build_dir(odoo_full, version=None):

    def check_4_release(travis_base_dir):
        lpath = False
        for ldir in LDIR:
            ldir = os.path.join(travis_base_dir, ldir)
            if (os.path.isdir(ldir) and
                    os.path.isfile('%s/release.py' % ldir)):
                lpath = ldir
                break
        return lpath

    VERSIONS = ('15.0', '14.0', '13.0', '12.0', '11.0',
                '10.0', '9.0', '8.0', '7.0', '6.1')
    items = odoo_full.split('/')
    if items[1] in VERSIONS:
        odoo_version = items[1]
    else:
        odoo_version = version or os.environ.get("VERSION")
    git_org = items[0]
    lpath = False
    if not odoo_version or odoo_version == "auto":
        for version in VERSIONS:
            travis_base_dir = z0testodoo.get_local_odoo_path(
                git_org, 'OCB', version, home=os.environ['HOME'])
            if travis_base_dir:
                lpath = check_4_release(travis_base_dir)
                if lpath:
                    break
    else:
        travis_base_dir = z0testodoo.get_local_odoo_path(
            git_org, 'OCB', odoo_version, home=os.environ['HOME'])
        if travis_base_dir:
            lpath = check_4_release(travis_base_dir)
    if lpath:
        sys.path.append(lpath)
        import release
        # This function may be called more with different odoo versions
        if sys.version_info[0] == 2:
            reload(release)
        else:
            importlib.reload(release)
        tested_version = release.version
        del release
        del sys.path[-1]
        if not odoo_version or odoo_version == "auto":
            odoo_version = tested_version
        elif odoo_version != tested_version:
            print_flush(
                "ERROR: invalid odoo %s version detected!" % tested_version)
        travis_base_dir = os.path.abspath('%s/addons' % lpath)
    else:
        print_flush("ERROR: no travis build dir detected!")
    if not odoo_version:
        print_flush("ERROR: no odoo version detected!")
    return travis_base_dir, odoo_version


# Return server script for all Odoo versions
def get_server_script(server_path):
    if os.path.isfile(get_script_path(server_path, 'odoo-bin')):
        return get_script_path(server_path, 'odoo-bin')
    elif os.path.isfile(get_script_path(server_path, 'openerp-server')):
        return get_script_path(server_path, 'openerp-server')
    return 'Script not found!'


def get_addons_to_check(travis_base_dir, odoo_include, odoo_exclude,
                        odoo_test_select, travis_debug_mode=None):
    """
    Get the list of modules that need to be installed
    :param travis_base_dir: Travis odoo core directory
    :param odoo_include: addons to include (travis parameter)
    :param odoo_exclude: addons to exclude (travis parameter)
    :param odoo module set to include/exclude
    :return: List of addons to test
    """
    if odoo_include:
        addons_list = parse_list(odoo_include)
    elif odoo_test_select != 'NO-CORE':
        addons_list = get_modules(travis_base_dir)
    else:
        addons_list = ['base']

    if odoo_exclude:
        exclude_list = parse_list(odoo_exclude)
        addons_list = [
            x for x in addons_list
            if x not in exclude_list]
        if travis_debug_mode > 1:
            print_flush('DEBUG: exclude_list=%s' % odoo_exclude)

    if odoo_test_select in ('APPLICATION', 'NO-APPLICATION',
                            'LOCALIZATION', 'NO-LOCALIZATION'):
        applications, localizations = set(), set()
        if odoo_test_select in ('APPLICATION', 'NO-APPLICATION'):
            applications = get_applications_with_dependencies(addons_list)
            if odoo_test_select == 'NO-APPLICATION':
                addons_list -= applications
                applications = set()
        if odoo_test_select in ('LOCALIZATION', 'NO-LOCALIZATION'):
            localizations = get_localizations_with_dependents(addons_list)
            if odoo_test_select == 'NO-LOCALIZATION':
                addons_list -= localizations
                localizations = set()
        if applications or localizations:
            addons_list = applications | localizations

    return addons_list


def get_test_dependencies(addons_path, addons_list):
    """
    Get the list of core and external modules dependencies
    for the modules to test.
    :param addons_path: string with a comma separated list of addons paths
    :param addons_list: list of the modules to test
    """
    if not addons_list:
        return ['base']
    else:
        modules = {}
        for path in addons_path.split(','):
            modules.update(get_modules_info(path))
        dependencies = set()
        for module in addons_list:
            dependencies |= get_dependencies(modules, module)
        return list(dependencies - set(addons_list))


def cmd_strip_secret(cmd):
    cmd_secret = []
    skip_next = False
    for param in cmd:
        if skip_next:
            skip_next = False
            continue
        if param.startswith('--db_'):
            cmd_secret.append(param.split('=')[0] + '=***')
            continue
        if param.startswith('--log-db'):
            cmd_secret.append('--log-db=***')
            continue
        if param in ['-w', '-r']:
            cmd_secret.extend([param, '***'])
            skip_next = True
            continue
        cmd_secret.append(param)
    return cmd_secret


# [antoniov: 2018-02-28]
def get_script_path(server_path, script_name):
    if (os.path.isdir(os.path.join(server_path, 'server')) and
            os.path.isfile(os.path.join(server_path, 'server', script_name))):
        script_path = os.path.join(server_path, 'server', script_name)
    elif (os.path.isdir(os.path.join(server_path, 'odoo')) and
            os.path.isfile(os.path.join(server_path, 'odoo', script_name))):
        script_path = os.path.join(server_path, 'odoo', script_name)
    else:
        script_path = os.path.join(server_path, script_name)
    return script_path


def build_run_cmd_odoo(server_path, script_name, db, modules=None,
                       install_options=None, server_options=None,
                       unbuffer=None, scope=None, test_loglevel=None,
                       coveragerc=None, port=None):
    # scope=('init','test')
    script_path = get_script_path(server_path, script_name)
    test_loglevel = test_loglevel or 'info'
    preinstall_modules = modules or []
    if os.environ.get('TRAVIS_PDB') == 'true':
        if scope == 'test':
            cmd_odoo = ["python", "-m", "pdb"]
        else:
            cmd_odoo = []
    elif scope == 'test':
        if coveragerc:
            cmd_odoo = ["python", "-m",
                        "coverage", "--rcfile", coveragerc, "run"]
        else:
            cmd_odoo = ["python", "-m", "coverage", "run"]
        if unbuffer:
            cmd_odoo.insert(0, "unbuffer")
    elif unbuffer:
        # unbuffer keeps output colors
        cmd_odoo = ["unbuffer"]
    else:
        cmd_odoo = []
    cmd_odoo += [script_path,
                 "-d", db,
                 "--db-filter=^%s$" % db,
                 "--log-level=%s" % test_loglevel,
                 "--stop-after-init",
                 ]
    if port:
        #TODO> PORT
        pass
    if install_options:
        cmd_odoo.append(install_options)
    if server_options:
        cmd_odoo.append(server_options)
    if scope == 'init':
        if preinstall_modules:
            cmd_odoo.append("--init=%s" % ','.join(preinstall_modules))
        else:
            cmd_odoo.append("--init")
            cmd_odoo.append(None)
    return cmd_odoo


def setup_server(db, server_path, script_name,
                 install_options, preinstall_modules=None,
                 unbuffer=True, server_options=None, travis_debug_mode=None):
    """
    Setup the base module before running the tests
    if the database template exists then will be used.
    :param db: Template database name
    :param odoo_unittest: Boolean for unit test (travis parameter)
    :param server_path: Server path
    :param script_name: Odoo start-up script (openerp-server or odoo-bin)
    :param addons_path: Addons path
    :param install_options: Install options (travis parameter)
    :param server_options: (list) Add these flags to the Odoo server init
    """
    travis_debug_mode = travis_debug_mode or 0
    if preinstall_modules is None:
        preinstall_modules = ['base']
    if server_options is None:
        server_options = []
    try:
        subprocess.check_call(["createdb", db])
        print_flush("INFO: database %s created." % db)
    except subprocess.CalledProcessError:
        print_flush("INFO: Using previous %s database." % db)
    else:
        cmd_odoo = build_run_cmd_odoo(
            server_path, script_name, db, modules=preinstall_modules,
            install_options=install_options,
            server_options=server_options, unbuffer=unbuffer,
            scope='init', test_loglevel=get_log_level_init(travis_debug_mode))
        print_flush('>>> %s\n' % " ".join(cmd_strip_secret(cmd_odoo)))
        if travis_debug_mode < 8:
            try:
                subprocess.check_call(cmd_odoo)
            except subprocess.CalledProcessError as e:
                return e.returncode
    return 9 - travis_debug_mode


def run_from_env_var(env_name_startswith, environ):
    """Method to run a script defined from an environment variable
    :param env_name_startswith: String with name of first letter of
                                environment variable to find.
    :param environ: Dictionary with full environ to search
    """
    commands = [
        command
        for environ_variable, command in sorted(environ.items())
        if environ_variable.startswith(env_name_startswith)
    ]
    for command in commands:
        print_flush("command: %s" % command)
        subprocess.call(command, shell=True)


def set_conf_data(addons_path, data_dir):
    conf_data = {
        'addons_path': addons_path,
        'data_dir': data_dir,
    }
    # [ antoniov: 2018-02-28 ]
    if os.environ.get('TRAVIS') != 'true':
        pid = os.getpid() % 65536
        if pid > 18000:
            rpcport = str(pid)
        else:
            rpcport = str(18000 + pid)
        conf_data['xmlrpc_port'] = rpcport
        conf_data['db_user'] = 'odoo'
    return conf_data


def create_server_conf(data, version):
    """Create (or edit) default configuration file of odoo
    :params data: Dict with all info to save in file"""
    if int(version.split('.')[0]) < 10:
        fname_conf = os.path.expanduser('~/.openerp_serverrc')
    else:
        fname_conf = os.path.expanduser('~/.odoorc')
    if not os.path.exists(fname_conf):
        # If not exists the file then is created
        with open(fname_conf, "w") as fconf:
            fconf.write('[options]\n')
    config = ConfigParser.ConfigParser()
    config.read(fname_conf)
    if not config.has_section('options'):
        config.add_section('options')
    for key, value in data.items():
        config.set('options', key, value)
    with open(fname_conf, 'w') as configfile:
        config.write(configfile)


def copy_attachments(dbtemplate, dbdest, data_dir):
    attach_dir = os.path.join(os.path.expanduser(data_dir), 'filestore')
    attach_tmpl_dir = os.path.join(attach_dir, dbtemplate)
    attach_dest_dir = os.path.join(attach_dir, dbdest)
    if os.path.isdir(attach_tmpl_dir) and not os.path.isdir(attach_dest_dir):
        print_flush("copy %s %s" % (attach_tmpl_dir, attach_dest_dir))
        shutil.copytree(attach_tmpl_dir, attach_dest_dir)


def get_environment():
    test_enable = str2bool(os.environ.get('TEST_ENABLE', True))
    options = os.environ.get("OPTIONS", "").split()
    server_options = os.environ.get('SERVER_OPTIONS', "").split()
    odoo_full = os.environ.get("ODOO_REPO", "odoo/odoo")
    travis_base_dir, odoo_version = get_build_dir(odoo_full)
    odoo_exclude = os.environ.get("EXCLUDE")
    gbl_exclude = os.environ.get("GBL_EXCLUDE")
    if gbl_exclude:
        odoo_exclude = '%s,%s' % (odoo_exclude, gbl_exclude)
    odoo_include = os.environ.get("INCLUDE")
    odoo_unittest = str2bool(os.environ.get("UNIT_TEST"))
    odoo_tnlbot = str2bool(os.environ.get('ODOO_TNLBOT', '0'))
    odoo_testdeps = str2bool(os.environ.get('TEST_DEPENDENCIES', '1'))
    install_options = os.environ.get("INSTALL_OPTIONS", "").split()
    travis_home = os.environ.get("HOME", os.path.expanduser("~"))
    travis_dependencies_dir = os.path.join(travis_home, 'dependencies')
    odoo_branch = os.environ.get("ODOO_BRANCH")
    data_dir = os.path.expanduser(
        os.environ.get("DATA_DIR", os.path.expanduser('~/data_dir')))
    odoo_test_select = os.environ.get('ODOO_TEST_SELECT', 'ALL')
    dbtemplate = os.environ.get('MQT_TEMPLATE_DB', 'openerp_template')
    database = os.environ.get('MQT_TEST_DB', 'openerp_test')
    travis_debug_mode = eval(os.environ.get('TRAVIS_DEBUG_MODE', '0'))
    if odoo_version == '6.1':
        unbuffer = str2bool(os.environ.get('UNBUFFER', False))
    else:
        unbuffer = str2bool(os.environ.get('UNBUFFER', True))
    set_sys_path()
    if travis_debug_mode:
        print_flush('DEBUG: test_server.sys.path=%s' % sys.path)

    server_path = get_server_path(odoo_full,
                                  odoo_branch or odoo_version,
                                  travis_home)
    script_name = get_server_script(server_path)
    if script_name != 'Script not found!':
        script_path = get_script_path(server_path, script_name)
        addons_path = get_addons_path(travis_dependencies_dir,
                                      travis_base_dir,
                                      server_path,
                                      odoo_test_select)
    else:
        script_path = ''
        addons_path = ''
    return (
        test_enable,
        options,
        server_options,
        odoo_full,
        odoo_version,
        travis_base_dir,
        odoo_exclude,
        odoo_include,
        odoo_unittest,
        odoo_tnlbot,
        odoo_testdeps,
        install_options,
        travis_home,
        travis_dependencies_dir,
        odoo_branch,
        server_path,
        script_name,
        odoo_test_select,
        addons_path,
        dbtemplate,
        database,
        data_dir,
        script_path,
        travis_debug_mode,
        unbuffer)


def set_coveragerc():
    coveragerc = os.path.join(
        os.path.abspath(os.path.join(__file__, 'cfg')), 'coveragerc')
    if os.path.isfile(coveragerc):
        return coveragerc
    return False


def get_log_level(odoo_version, test_enable, travis_debug_mode, tnlbot=False):
    test_loghandler = None
    options = []
    if tnlbot:
        if odoo_version == "6.1" and not test_enable:
            options = ["--test-disable"]
        test_loglevel = 'info'
    elif odoo_version == "6.1":
        if not test_enable:
            options = ["--test-disable"]
        test_loglevel = 'test'
    else:
        if test_enable:
            options = ["--test-enable"]
        if odoo_version == '7.0':
            test_loglevel = 'test'
        elif travis_debug_mode > 1:
            test_loglevel = 'test'
        else:
            test_loglevel = 'info'
            test_loghandler = 'openerp.tools.yaml_import:DEBUG'
    return options, test_loglevel, test_loghandler


def get_log_level_init(travis_debug_mode):
    return {
        0: 'error',
        1: 'warn'
    }.get(travis_debug_mode, 'info')


def set_sys_path():
    x = -1
    for i in range(len(sys.path)):
        x = i if sys.path[i].endswith('/tools') else x
    if x > 0:
        sys.path.insert(0, sys.path[x])
        del sys.path[x + 1]
    else:
        sys.path.insert(0, os.path.join(os.environ.get('HOME'), 'tools'))


def main(argv=None):
    run_from_env_var('RUN_COMMAND_MQT', os.environ)
    expected_errors = int(os.environ.get("SERVER_EXPECTED_ERRORS", "0"))
    instance_alive = str2bool(os.environ.get('INSTANCE_ALIVE'))
    # odoo_test_select = os.environ.get('ODOO_TEST_SELECT', 'ALL')
    (test_enable, options, server_options,
     odoo_full, odoo_version, travis_base_dir,
     odoo_exclude, odoo_include,
     odoo_unittest, odoo_tnlbot, odoo_testdeps,
     install_options, travis_home,
     travis_dependencies_dir, odoo_branch,
     server_path, script_name, odoo_test_select,
     addons_path, dbtemplate, database,
     data_dir, script_path, travis_debug_mode,
     unbuffer) = get_environment()
    opts, test_loglevel, test_loghandler = get_log_level(odoo_version,
                                                         test_enable,
                                                         travis_debug_mode)
    options += opts
    if travis_debug_mode:
        print_flush("DEBUG: server_path='%s'" % server_path)
        print_flush("DEBUG: script_name='%s'" % script_name)
    if script_name == 'Script not found!':
        print_flush("ERROR: %s!" % script_name)
        return 1
    coveragerc = set_coveragerc()
    conf_data = set_conf_data(addons_path, data_dir)
    create_server_conf(conf_data, odoo_version)
    tested_addons_list = get_addons_to_check(
        travis_base_dir,
        odoo_include,
        odoo_exclude,
        odoo_test_select,
        travis_debug_mode=travis_debug_mode)
    tested_addons = ','.join(tested_addons_list)

    print_flush("INFO: Working in %s" % travis_base_dir)
    print_flush("INFO Using repo %s and addons path %s" % (
        odoo_full, addons_path))

    if not tested_addons:
        print_flush("WARNING!\nNothing to test- exiting early.")
        return 0
    else:
        print_flush("INFO: modules to test: %s" % tested_addons_list)
    # setup the preinstall modules without running the tests
    preinstall_modules = get_test_dependencies(addons_path,
                                               tested_addons_list)

    preinstall_modules = list(set(preinstall_modules or []) - set(get_modules(
        os.environ.get('TRAVIS_BUILD_DIR')) or [])) or ['base']
    print_flush("INFO: modules to preinstall: %s\n" % preinstall_modules)
    setup_server(dbtemplate, server_path, script_name,
        install_options, preinstall_modules,
        unbuffer, server_options, travis_debug_mode)
    cmd_odoo_test = build_run_cmd_odoo(
        server_path, script_name, database, unbuffer=unbuffer,
        scope='test', test_loglevel=test_loglevel, coveragerc=coveragerc,
        port=conf_data.get('xmlrpc_port'))

    if test_loghandler is not None:
        cmd_odoo_test += ['--log-handler', test_loghandler]
    cmd_odoo_test += options + ["--init", None]

    if odoo_testdeps:
        to_test_list = [tested_addons]
        commands = (('travis_test_dependencies', True),
                    )
    if odoo_unittest:
        to_test_list = tested_addons_list
        cmd_odoo_install = build_run_cmd_odoo(
            server_path, script_name, database,
            install_options=install_options,
            server_options=server_options, unbuffer=unbuffer,
            scope='init', test_loglevel=get_log_level_init(travis_debug_mode))
        commands = ((cmd_odoo_install, False),
                    (cmd_odoo_test, True),
                    )
    else:
        to_test_list = [tested_addons]
        commands = ((cmd_odoo_test, True),
                    )

    all_errors = []
    counted_errors = 0
    for to_test in to_test_list:
        print_flush("\nTEST: %s:" % to_test)
        # db_odoo_created = False
        try:
            db_odoo_created = subprocess.call(
                ["createdb", "-T", dbtemplate, database])
            copy_attachments(dbtemplate, database, data_dir)
        except subprocess.CalledProcessError:
            db_odoo_created = True
        for command, check_loaded in commands:
            if db_odoo_created and instance_alive:
                # If exists database of odoo test
                # then start server with regular command without tests params
                rm_items = [
                    'python', '-m', 'coverage', 'run', '--stop-after-init',
                    '--test-enable', '--init', None,
                    '--log-handler', 'openerp.tools.yaml_import:DEBUG',
                ]
                command_call = [item
                                for item in commands[0][0]
                                if item not in rm_items] + \
                    ['--pidfile=/tmp/odoo.pid']     # 4
            else:
                command[-1] = to_test
                command_call = command
            if travis_debug_mode >= 8:
                if int(odoo_version.split('.')[0]) < 10:
                    fname_conf = os.path.expanduser('~/.openerp_serverrc')
                else:
                    fname_conf = os.path.expanduser('~/.odoorc')
                print_flush('>>> cat %s' % fname_conf)
                with open(fname_conf, 'r') as fd:
                    print_flush(fd.read())
                print_flush(
                    '\n>>> %s' % " ".join(cmd_strip_secret(command_call)))
                errors = 9 - travis_debug_mode
            else:
                print_flush(
                    '>>> %s' % " ".join(cmd_strip_secret(command_call)))
                pipe = subprocess.Popen(command_call,
                                        stderr=subprocess.STDOUT,
                                        stdout=subprocess.PIPE)
                with open('stdout.log', 'wb') as stdout:
                    for line in iter(pipe.stdout.readline, b''):
                        stdout.write(line)
                        print(line.strip().decode('UTF-8'))
                returncode = pipe.wait()
                # Find errors, except from failed mails
                errors = has_test_errors(
                    "stdout.log", database, odoo_version, check_loaded)
                if returncode != 0:
                    all_errors.append(to_test)
                    print(fail_msg, "Command exited with code %s" % returncode)
                    # If not exists errors then
                    # add an error when returcode!=0
                    # because is really a error.
                    if not errors:
                        errors += 1
            if errors:
                counted_errors += errors
                all_errors.append(to_test)
                print(fail_msg, "Found %d lines with errors" % errors)
        if not instance_alive and odoo_unittest:
            # Don't drop the database if will be used later.
            subprocess.call(["dropdb", database])

    print('Module test summary')
    for to_test in to_test_list:
        if to_test in all_errors:
            print(fail_msg, to_test)
        else:
            print(success_msg, to_test)
    if expected_errors and counted_errors != expected_errors:
        print("Expected %d errors, found %d!"
              % (expected_errors, counted_errors))
        return 1
    elif counted_errors != expected_errors:
        return 1

    return 0


if __name__ == '__main__':
    exit(main())



# -*- coding: utf-8 -*-

import subprocess
from six import string_types


class GitRun(object):

    def __init__(self, repo_path, debug=False):
        self.repo_path = repo_path
        self.debug = debug

    def run(self, command):
        """Execute git command in bash
        :param list command: Git cmd to execute in self.repo_path
        :return: String output of command executed.
        """
        cmd = ['git', '--git-dir=' + self.repo_path] + command
        print(cmd if self.debug else '')
        try:
            res = subprocess.check_output(cmd)
        except subprocess.CalledProcessError:
            res = None
        if isinstance(res, string_types):
            res = res.strip('\n')
        return res

    def get_items_changed(self, base_ref='HEAD'):
        """Get name of items changed in self.repo_path
        This is a wrapper method of git command:
            git diff-index --name-only --cached {base_ref}
        :param base_ref: String of branch or sha base.
            e.g. "master" or "SHA_NUMBER"
        :return: List of name of items changed
        """
        command = ['diff-index', '--name-only',
                   '--cached', base_ref]
        res = self.run(command)
        items = res.decode('UTF-8').split('\n') if res else []
        return items

    def get_branch_name(self):
        """Get branch name
        :return: String with name of current branch name"""
        command = ['rev-parse', '--abbrev-ref', 'HEAD']
        res = self.run(command)
        return res



# -*- coding: utf-8 -*-
from setuptools import setup
from setuptools import find_packages

setup(name='z0bug_odoo',
      version='1.0.11',
      description='Odoo testing framework',
      long_description="""
Zeroincombenze(R) continuous testing framework for Odoo modules.

Make avaiable test functions indipendent by Odoo version.
""",
      classifiers=[
          'Development Status :: 4 - Beta',
          'License :: OSI Approved :: GNU Affero General Public License v3',
          'Operating System :: POSIX',
          'Programming Language :: Python :: 2.7',
          'Programming Language :: Python :: 3.6',
          'Programming Language :: Python :: 3.7',
          'Intended Audience :: Developers',
          'Topic :: Software Development',
          'Topic :: Software Development :: Libraries',
          'Topic :: System :: System Shells',
      ],
      keywords='unit test debug',
      url='https://zeroincombenze-tools.readthedocs.io',
      project_urls={
          'Documentation': 'https://zeroincombenze-tools.readthedocs.io',
          'Source': 'https://github.com/zeroincombenze/tools',
      },
      author='Antonio Maria Vigliotti',
      author_email='antoniomaria.vigliotti@gmail.com',
      license='Affero GPL',
      install_requires=['future', 'python-magic', 'zerobug'],
      packages=find_packages(
          exclude=['docs', 'examples', 'tests', 'egg-info', 'junk']),
      package_data={
          '': ['scripts/setup.info', 'data/*', 'travis/cfg/*',
               'travis/pylint_deprecated_modules/*',
               'travis/pylint_deprecated_modules/openerp/*',
               'travis/pylint_deprecated_modules/openerp/osv/*',
               'travis/clone_oca_dependencies',
               'travis/test_flake8',
               'travis/test_pylint',
               'travis/travis_makepot',
               'travis/travis_run_flake8',
               'travis/travis_run_tests',
               'travis/travis_test_dependencies',
               ],
      },
      entry_points={
          'console_scripts': [
              'z0bug-odoo-info = z0bug_odoo.scripts.main:main',
              # 'travis_run_tests = z0bug_odoo.travis.travis_run_tests:main',
          ],
      },
      zip_safe=False)



"""
Odoo connection classes which describe how to connect to odoo to export PO
files.
One class per version is defined and mapped in context_mapping.
To add a new version, create a subclass of _OdooBaseContext with name
OdooXContext, implement __enter__ and add to context_mapping.
"""

import sys
from contextlib import closing


class _OdooBaseContext(object):
    """
    Abstract class for odoo connections and translation export without
    version specification.
    Inherit from this class to implement version specific connection
    parameters.
    """

    def __init__(self, server_path, addons_path, dbname):
        """
        Create context object. Stock odoo server path and database name.
        :param str server_path: path to odoo root
        :param str addons_path: comma separated list of addon paths
        :param str dbname: database name with odoo installation
        """
        self.server_path = server_path
        self.addons_path = addons_path
        self.dbname = dbname

    def __enter__(self):
        raise NotImplementedError("The class %s is an abstract class which"
                                  "doesn't have __enter__ implemented."
                                  % self.__class__.__name__)

    def __exit__(self, exc_type, exc_val, exc_tb):
        """
        Cleanly close cursor
        """
        self.cr.close()

    def get_pot_contents(self, addon, lang=None):
        """
        Export source translation files from addon.
        :param str addon: Addon name
        :returns str: Gettext from addon .pot content
        """
        import cStringIO
        import codecs
        buffer = cStringIO.StringIO()
        codecs.getwriter("utf8")(buffer)
        self.trans_export(lang, [addon], buffer, 'po', self.cr)
        tmp = buffer.getvalue()
        buffer.close()
        return tmp

    def load_po(self, po, lang):
        self.trans_load_data(self.cr, po, 'po', lang)


class Odoo10Context(_OdooBaseContext):
    """A context for connecting to a odoo 10 server with function to export
    .pot files.
    """

    def __enter__(self):
        """
        Context enter function.
        Temporarily add odoo 10 server path to system path and pop afterwards.
        Import odoo 10 server from path as library.
        Init logger, registry and environment.
        Add addons path to config.
        :returns Odoo10Context: This instance
        """
        sys.path.append(self.server_path)
        from odoo import netsvc, api
        from odoo.modules.registry import Registry
        from odoo.tools import trans_export, config, trans_load_data
        self.trans_export = trans_export
        self.trans_load_data = trans_load_data
        sys.path.pop()
        netsvc.init_logger()
        config['addons_path'] = (
            config.get('addons_path') + ',' + self.addons_path
        )
        registry = Registry.new(self.dbname)
        self.environment_manage = api.Environment.manage()
        self.environment_manage.__enter__()
        self.cr = registry.cursor()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        """
        Context exit function.
        Cleanly close environment manage and cursor.
        """
        self.environment_manage.__exit__(exc_type, exc_val, exc_tb)
        super(Odoo10Context, self).__exit__(exc_type, exc_val, exc_tb)


class Odoo11Context(Odoo10Context):
    """A context for connecting to a odoo 11 server with an special override
    for getting translations with Python 3.
    """
    def get_pot_contents(self, addon, lang=None):
        """
        Export source translation files from addon.
        :param str addon: Addon name
        :returns bytes: Gettext from addon .pot content
        """
        from io import BytesIO
        with closing(BytesIO()) as buf:
            self.trans_export(lang, [addon], buf, 'po', self.cr)
            return buf.getvalue()


class Odoo8Context(_OdooBaseContext):
    """
    A context for connecting to a odoo 8 server with function to export .pot
    """

    def __enter__(self):
        """
        Context enter function.
        Temporarily add odoo 8 server path to system path and pop afterwards.
        Import odoo 8 server from path as library.
        Init logger, registry and environment.
        Add addons path to config.
        :returns Odoo8Context: This instance
        """
        sys.path.append(self.server_path)
        from openerp import netsvc, api
        from openerp.modules.registry import RegistryManager
        from openerp.tools import trans_export, config, trans_load_data
        self.trans_export = trans_export
        self.trans_load_data = trans_load_data
        sys.path.pop()
        netsvc.init_logger()
        config['addons_path'] = (
            config.get('addons_path') + ',' + self.addons_path
        )
        registry = RegistryManager.new(self.dbname)
        self.environment_manage = api.Environment.manage()
        self.environment_manage.__enter__()
        self.cr = registry.cursor()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        """
        Context exit function.
        Cleanly close environment manage and cursor.
        """
        self.environment_manage.__exit__(exc_type, exc_val, exc_tb)
        super(Odoo8Context, self).__exit__(exc_type, exc_val, exc_tb)


class Odoo7Context(_OdooBaseContext):
    """
    A context for connecting to a odoo 7 server with function to export .pot
    """

    def __enter__(self):
        """
        Context enter function.
        Temporarily add odoo 7 server path to system path and pop afterwards.
        Import odoo 7 server from path as library.
        Init logger and pool.
        Add addons path to config.
        :returns Odoo8Context: This instance
        """
        sys.path.append(self.server_path)
        from openerp import netsvc
        from openerp.tools import trans_export, config, trans_load_data
        from openerp.pooler import get_db
        self.trans_export = trans_export
        self.trans_load_data = trans_load_data
        sys.path.pop()
        netsvc.init_logger()
        config['addons_path'] = str(
            config.get('addons_path') + ',' + self.addons_path
        )
        self.cr = get_db(self.dbname).cursor()
        return self


context_mapping = {
    "7.0": Odoo7Context,
    "8.0": Odoo8Context,
    "9.0": Odoo8Context,
    "10.0": Odoo10Context,
    "11.0": Odoo11Context,
}
